/* 
 * Plugin name: Find & Replace
 * Plugin description: Calculates segment rate using a moving average method. A description of an application of this plugin to syllable-based annotations can be found in a paper by M. Karpiński, K. Klessa, A. Czoska (Proc. Speech Prosody, 2014).
 * Plugin Author: Katarzyna Klessa
 * Plugin Created: 2014-04-20
 * 
 * Plugin Id: 748dcaa4
 * Plugin Version: 1.0
 * 
 */



using System;
using System.Windows.Forms;
using AnnotationPro.Plugin;
using AnnotationPro.Logic;
using AnnotationPro.Presentation;

namespace AnnotationPro.Plugin
{
  public class AnnotationPlugin : IAnnotationPlugin
  {
    public void Run( AnnotationEditor editor )
    {
      // BEGIN USER CODE

      string layerName = "SRC";
      float frameWidth = 10;
      float frameStep = frameWidth * 2 / 3; // 33%

      Annotation annotation = editor.Synchronizer.DataServer.Annotation;
      Audio audio = editor.Synchronizer.DataServer.Audio;

      LayerObject srcLayer = null;
      foreach ( LayerObject layer in annotation.Layers ) {
        if ( layer.Name == layerName ) {
          srcLayer = layer;
          break;
        }
      }
      //LayerObject regionsLayer = annotation.Layers.FindByName( "Regions" );

      if ( srcLayer == null ) {
        MessageBox.Show( "Missing layer: " + layerName );
        return;
      }
      //creating new layer for storing the results of calculation, the layer name is the regions layer name - SRMA
      //adding the parameter names - results will be stored as parameters
      LayerObject dstLayer = editor.AnnotationLayers.AddNewLayer();
      dstLayer.Name = srcLayer.Name + " - SRMA";
      dstLayer.Parameter1Name = "FrameStart + FrameEnd";
      dstLayer.Parameter2Name = "Count + CountWithFractions";
      dstLayer.Parameter3Name = "Count/FrameWidth + CountWithFractions/FrameWidth + CountWithFractions/TotalDuration";

      float maxTime = audio.SamplesCount / audio.Samplerate;
      int frameIndex = 0;
      for ( float frameOffset = 0; frameOffset < maxTime; frameOffset += frameStep ) {
        float frameStart = frameOffset * annotation.Samplerate;
        float frameEnd = (frameOffset + frameWidth) * annotation.Samplerate;
        float frameMiddle = (frameOffset + frameWidth / 2) * annotation.Samplerate;
        
        // count the number and duration of segments within a frame
        float count = 0;
        float countWithFractions = 0;
        float totalDuration = 0;

        foreach ( SegmentObject segment in srcLayer.Segments ) {
          if ( segment.End <= frameStart || segment.Start > frameEnd ) continue;
          if ( segment.Label == "#$p" ) continue; // each line excludes a label from calculations
          if ( segment.Label == "sil" ) continue;
          if ( segment.Label == "..." ) continue;
          count++;
          // count the number with fractions (including information of segments partially within the frame range) and duration of segments within a frame
          float intersection = GetSegmentIntersection( segment, frameStart, frameEnd );
          float fraction = intersection / segment.Duration;

          countWithFractions += fraction;
          totalDuration += intersection / annotation.Samplerate;
        }

        // adding frame segment with results of calculations, the width of the segment is 0.5 of the frame width
        SegmentObject dstSegment = new SegmentObject();
        dstSegment.Start = frameStart;
        dstSegment.Duration = 0.5f * annotation.Samplerate;

        dstSegment.Label = "Frame #" + frameIndex + " Count: " + count.ToString() + ", Count (FRACT): " + countWithFractions.ToString() + ", Total Dur: " + totalDuration.ToString() + ", Frame Width:" + frameWidth;
        float frameOffsetPlusWidth = frameOffset+frameWidth;
        dstSegment.Parameter1 = frameOffset.ToString() + "+" + frameOffsetPlusWidth.ToString();
        dstSegment.Parameter2 = count.ToString() + "+" + countWithFractions.ToString();
        float srma = count / frameWidth;
        float srma2 = countWithFractions / frameWidth;
        float srma3 = countWithFractions / totalDuration;
        dstSegment.Parameter3 = srma.ToString() + "+" + srma2.ToString() + "+" + srma3.ToString() ; 

        dstLayer.Segments.Add( dstSegment );

        frameIndex++;
      }
      
      // END USER CODE
    }

    public float GetSegmentIntersection( SegmentObject segment, float rangeStart, float rangeEnd ) {
      if ( segment.Start < rangeStart && segment.End > rangeStart && segment.End <= rangeEnd ) {
        return segment.End - rangeStart;
      }
      if ( segment.Start < rangeEnd && segment.Start > rangeStart && segment.End > rangeEnd ) {
        return rangeEnd - segment.Start;
      }
      if ( segment.Start > rangeStart && segment.End < rangeEnd ) {
        return segment.Duration;
      }
      if ( segment.Start < rangeStart && segment.End > rangeEnd ) {
        return rangeEnd - rangeStart;
      }
      
      return 0;
    }
  }
}